;;
;; PHYSICS SYSTEM
;;

.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "cpctelera.h.s"
.include "man/game.h.s"

.module sys_entity_collisions

muerto: .asciz "Game Over"
press1: .asciz "Press any key "
press2: .asciz "to restart"

screen_width = 80
screen_height = 200
max_player = 80
door: DefineCmp_Entity  78, 90, 0, 0, 2, 16, e_color_door, e_ai_st_stand_by, 1, e_type_puerta
sys_collisions_init::

	ld (_ent_array_ptr_temp_standby), hl
	ld (_ent_array_ptr_temp_enemy_collisions), hl
	ld (_ent_array_ptr_check_all_enemys_dead), hl
	ld (_ent_array_ptr_player), hl
	ret



;;MOVE PLAYER BUT WITH MAX
;; IX: Pointer to entity array
;;
sys_player_max::
	ld a, e_type(ix)
	cp #e_type_player
	jr z, player
	ret

player:
	ld a, #max_player
	sub e_w(ix)
	ld c,a

	ld a, e_x(ix)
	add e_vx(ix)
	cp c
	jr nc, invalid_x

valid_x:
	ld e_x(ix), a
	jr endif_x
;;Bounces
invalid_x:
	ld a, e_vx(ix)
	neg
	call comprobar_puerta
	ld e_vx(ix), a
	jr endif_x

endif_x:
	ld a, #screen_height + 1
	sub e_h(ix)
	ld c,a 
	
	ld a, e_y(ix)
	add e_vy(ix)
	cp c
	jr nc, invalid_y
valid_y:
	ld e_y(ix), a
	jr endif_y
invalid_y:
	ld a, e_vy(ix)
	neg
	ld e_vy(ix), a
endif_y:
	ret

;;
;; MOVE REST OF ENTITIES
;;
sys_move_rest::
	ld a, e_type(ix)
	cp #e_type_player
	jr nz, rest
	ret

rest:

	cp #e_type_enemy_collider
	jr nz, move_resttt
_ent_array_ptr_player = . + 2
	ld iy, #0x0000

	ld a, e_x(iy)
	ld e_ai_aim_x(ix), a


	ld a, e_y(iy)
	ld e_ai_aim_y(ix), a

	ld e_ai_st(ix), #e_ai_st_move_to

move_resttt:
	ld a, #screen_width + 1
	sub e_w(ix)
	ld c,a

	ld a, e_x(ix)
	add e_vx(ix)
	ld e_x(ix), a

endif_rest_x::
	ld a, #screen_height + 1
	sub e_h(ix)
	ld c,a 
	
	ld a, e_y(ix)
	add e_vy(ix)
	cp c
	jr nc, invalid_rest_y
valid_rest_y:
	ld e_y(ix), a
	jr endif_rest_y
invalid_rest_y:
	ld a, e_vy(ix)
	neg
	ld e_vy(ix), a
endif_rest_y:
	ret
;;
;; CHECKS COLLISIONS OF SHOOT BULLET 
;; AND IF HITS BACK OF WINDOWS ITS ERASED
;; IX: points to bullet
;; hl: first element
sys_check_bullet_collisions::
	ld a, e_type(ix)
	cp #e_type_bullet
	jr z, check_bullet
	ret
	
;;Check if hit back of window
check_bullet:
_ent_array_ptr_temp_standby = . + 2
	ld iy, #0x0000

	ld a, #screen_width -1
	sub e_w(ix)
	sub e_x(ix)
	jr c, destroy_bullet

	jr _update_loop
;;Destroy bullet
destroy_bullet:
	ld e_type(ix), #e_type_usable
	ld e_vx(ix), #0
	ld e_vy(ix), #0
	ret

;;Check collisions bullet with enemies
_update_loop:
	ld a, e_type(iy)
	cp #e_type_bullet
	jr z, end
	cp #e_type_usable
	jr z, end
	cp #e_type_puerta
	jr z, end

	ld a, e_w(iy)
	cp #e_w_invalidEntity
	ret z

	
check_hit_bullet::
	;;Colisiones eje x
	ld a, e_x(iy)
	add e_w(iy) 
	sub e_x(ix)
	jr c, end

	;;Por detrás(puede ser ineficiente e inutil)
	ld a, e_x(ix)
	add e_w(ix) 
	sub e_x(iy)
	jr c, end

	;;Colision eje y
	ld a, e_y(iy)
	add e_h(iy) 
	sub e_y(ix)
	jr c, end

	;;Colisiones por abajo
	ld a, e_y(ix)
	add e_h(ix) 
	sub e_y(iy)
	jr c, end
	
	call destroy_bullet


;;Remove life of entity hit
	ld a, e_lifes(iy)
	sub #1
	cp #0
	jr z, remove_entity
	ld e_lifes(iy), a
	ret

;;Remove entity
remove_entity::
	ld a, e_type(iy)
	cp #e_type_player
	call z, player_dead
	ld e_type(iy), #e_type_usable
	max_player = screen_width
	;;Cualquier otra entidad comprobar que todos muertos entonces aparece comprobar_puert
_ent_array_ptr_check_all_enemys_dead = . + 2
	ld ix, #0x0000

buscar_enemigos::
	ld a, e_w(ix)
	cp #e_w_invalidEntity
	jr nz, checkear_enemigos
	ld hl, #door
	call man_entity_create
	ret
checkear_enemigos:

	ld a, e_type(ix)
	cp #e_type_enemy
	ret z

	ld a, e_type(ix)
	cp #e_type_enemy_collider
	ret z

	ld de, #sizeof_e
	add ix, de

	jr buscar_enemigos

end:
	
	ld de, #sizeof_e
	add iy, de

	call _update_loop

;;
;; Check enemy colliders with player
;;
;; IX: enemy type collider to check
;; HL: first element(player)
sys_check_enemy_collisions::

_ent_array_ptr_temp_enemy_collisions = . + 2
	ld iy, #0x0000
	
	ld a, e_type(ix)
	cp #e_type_enemy_collider
	jr z, check_collider_hit_front
	ret
check_collider_hit_front:

	ld a, #0
	ld c,a

	ld a, e_x(ix)

	cp c
	jr c, hit_front
	jr check_hit_player

;;Destroy enemy collider
hit_front:
	ld e_type(ix), #e_type_usable
	ret
;;IY: player
;;IX: enemy collider
check_hit_player:
	
	ld a, e_x(iy)
	add e_w(iy) 
	sub e_x(ix)
	ret c

	ld a, e_x(ix)
	add e_w(ix) 
	sub e_x(iy)
	ret c


	ld a, e_y(iy)
	add e_h(iy) 
	sub e_y(ix)
	ret c


	ld a, e_y(ix)
	add e_h(ix) 
	sub e_y(iy)
	ret c
	
;;Remove life of entity and collider entity
player_hit:
	ld a, e_lifes(iy)
	sub #1
	;;Check if no lifes
	cp #0
	jr z, remove_player_entity
	ld e_type(ix), #e_type_usable

	;;Remove one life
	ld e_lifes(iy), a

	ret

;;Remove entity
remove_player_entity:

	cp #e_type_player
	jr z, player_dead
	call inc_score
	ret
	

player_dead:
	call borrar_mapa
	halt
	call has_muerto
	halt
	call borrar_mapa
	ld a, #1
	ld (mapa),a
	call reset_score
	call man_game_end
 	
 	ret

has_muerto::
	ld	HL,#0xC335
	ld	IY, #muerto

	call cpct_drawStringM0_asm

	ld	HL,#0xC3d0
	ld	IY, #press1

	call cpct_drawStringM0_asm

	ld	HL,#0xC425
	ld	IY, #press2

	call cpct_drawStringM0_asm

	call cpct_scanKeyboard_asm
bucle:
	call cpct_isAnyKeyPressed_asm
	cp #0
	jp nz, fin1
	
	
	jp has_muerto
fin1: 
ret

comprobar_puerta:
	;ld a, (mapa)
	;cp #1
	;jp nz,fin

	call man_entity_getArray
	ld a, e_x(ix)
	ld c, #77

	cp c
	jr z, comprobar_y

	ret
comprobar_y:
	call man_entity_getArray
	ld a, e_y(ix)

	ld c, #90
	cp c
	jr z, cambio_mapa

	ld c, #94
	cp c
	jr z, cambio_mapa

	ld c, #96
	cp c
	jr z, cambio_mapa

	ld c, #98
	cp c
	jr z, cambio_mapa

	ld c, #102
	cp c
	jr z, cambio_mapa

	ld c, #106
	cp c
	jr z, cambio_mapa

	ld c, #110
	cp c
	jr z, cambio_mapa

	ld c, #114
	cp c
	jr z, cambio_mapa
ret
cambio_mapa:
	call inc_score
	ld a, (mapa)
	add #1
	ld (mapa),a
	call empezar_mapa
		ret
fin:

	ret

